@@ -1,12 +1,11 @@
 base.bal.tab <- function(X, ...) {
     UseMethod("base.bal.tab")
 }
-base.bal.tab.base <- function(X, type, int = FALSE, poly = 1, continuous, binary, imbalanced.only = getOption("cobalt_imbalanced.only", FALSE), un = getOption("cobalt_un", FALSE), disp = NULL, disp.bal.tab = getOption("cobalt_disp.bal.tab", TRUE), abs = FALSE, quick = TRUE, ...) {
+base.bal.tab.base <- function(X, type, int = FALSE, poly = 1, continuous, binary, imbalanced.only = getOption("cobalt_imbalanced.only", FALSE), un = getOption("cobalt_un", FALSE), disp = NULL, disp.bal.tab = getOption("cobalt_disp.bal.tab", TRUE), disp.call = getOption("cobalt_disp.call", TRUE), abs = FALSE, quick = TRUE, ...) {
     #Preparations
-    
+
     A <- clear_null(list(...))
     A$subset <- NULL
-    
     X$treat <- process_treat(X$treat) 
     
     if (type == "bin") {
@@ -37,12 +36,7 @@ base.bal.tab.base <- function(X, type, int = FALSE, poly = 1, continuous, binary
     disp <- do.call("process_disp", c(list(disp), A), quote = TRUE)
     
     #Actions
-    out.names <- c("Balance", 
-                   expand.grid_string(c("Balanced", "Max.Imbalance"),
-                                      X$stats,
-                                      collapse = "."), 
-                   "Observations", "call")
-    out <- make_list(out.names)
+    out <- list()
     
     C <- do.call("get.C2", c(X, A[names(A) %nin% names(X)], list(int = int, poly = poly)), quote = TRUE)
     
@@ -62,41 +56,11 @@ base.bal.tab.base <- function(X, type, int = FALSE, poly = 1, continuous, binary
     thresholds <- attr(out[["Balance"]], "thresholds")
     disp <- attr(out[["Balance"]], "disp")
     
-    for (s in compute) {
-        if (is_not_null(thresholds[[s]])) {
-            if (no.adj) {
-                out[[paste.("Balanced", s)]] <- baltal(out[["Balance"]][[paste.(STATS[[s]]$Threshold, "Un")]])
-                out[[paste.("Max.Imbalance", s)]] <- max.imbal(out[["Balance"]][out[["Balance"]][["Type"]]!="Distance", , drop = FALSE], 
-                                                               col.name = paste.(STATS[[s]]$bal.tab_column_prefix, "Un"), 
-                                                               thresh.col.name = paste.(STATS[[s]]$Threshold, "Un"), 
-                                                               abs_stat = STATS[[s]]$abs)
-            }
-            else if (ncol(X$weights) == 1) {
-                out[[paste.("Balanced", s)]] <- baltal(out[["Balance"]][[STATS[[s]]$Threshold]])
-                out[[paste.("Max.Imbalance", s)]] <- max.imbal(out[["Balance"]][out[["Balance"]][["Type"]]!="Distance", , drop = FALSE], 
-                                                               col.name = paste.(STATS[[s]]$bal.tab_column_prefix, "Adj"), 
-                                                               thresh.col.name = STATS[[s]]$Threshold, 
-                                                               abs_stat = STATS[[s]]$abs)
-            }
-            else if (ncol(X$weights) > 1) {
-                out[[paste.("Balanced", s)]] <- setNames(do.call("cbind", lapply(names(X$weights), function(x) baltal(out[["Balance"]][[paste.(STATS[[s]]$Threshold, x)]]))),
-                                                         names(X$weights))
-                out[[paste.("Max.Imbalance", s)]] <- cbind(Weights = names(X$weights),
-                                                           do.call("rbind", lapply(names(X$weights), function(x) setNames(max.imbal(out[["Balance"]][out[["Balance"]][["Type"]]!="Distance", , drop = FALSE], 
-                                                                                                                                    col.name = paste.(STATS[[s]]$bal.tab_column_prefix, x), 
-                                                                                                                                    thresh.col.name = paste.(STATS[[s]]$Threshold, x), 
-                                                                                                                                    abs_stat = STATS[[s]]$abs),
-                                                                                                                          c("Variable", 
-                                                                                                                            STATS[[s]]$bal.tab_column_prefix, 
-                                                                                                                            STATS[[s]]$Threshold)))),
-                                                           stringsAsFactors = FALSE)
-            }
-        }
-        else {
-            out[[paste.("Balanced", s)]] <- NULL
-            out[[paste.("Max.Imbalance", s)]] <- NULL
-        }
-    }
+    out <- c(out, threshold.summary(compute = compute,
+                                    thresholds = thresholds,
+                                    no.adj = no.adj,
+                                    balance.table = out[["Balance"]],
+                                    weight.names = names(X$weights)))
     
     out[["Observations"]] <- samplesize(treat = X$treat, type = type, weights = X$weights, s.weights = X$s.weights, method = X$method, discarded = X$discarded)
     
@@ -107,14 +71,15 @@ base.bal.tab.base <- function(X, type, int = FALSE, poly = 1, continuous, binary
                                        compute = compute, 
                                        disp = disp,
                                        disp.adj=!no.adj,
-                                       disp.bal.tab = disp.bal.tab, 
+                                       disp.bal.tab = disp.bal.tab,
+                                       disp.call = disp.call, 
                                        abs = abs,
                                        continuous = continuous,
                                        binary = binary,
                                        quick = quick,
                                        nweights = ifelse(no.adj, 0, ncol(X$weights)),
                                        weight.names = names(X$weights),
-                                       treat_names = treat_vals(X$treat),
+                                       treat_names = treat_names(X$treat),
                                        type = type,
                                        co.names = co.names)
     class(out) <- c(paste.("bal.tab", type), "bal.tab")
@@ -131,7 +96,7 @@ base.bal.tab.cont <- function(X, ...) {
 base.bal.tab.imp <- function(X, which.imp = NA, imp.summary = getOption("cobalt_imp.summary"), imp.fun = getOption("cobalt_imp.fun", NULL), ...) {
     A <- list(...)
     
-    X$treat <- process_treat(X$treat)
+    # X$treat <- process_treat(X$treat)
     
     #Preparations
     
@@ -149,7 +114,9 @@ base.bal.tab.imp <- function(X, which.imp = NA, imp.summary = getOption("cobalt_
                  (is.numeric(which.imp) && !any(which.imp %in% seq_len(nlevels(imp)))))
     }
     
-    imp.fun <- if_null_then(imp.fun, A[["agg.fun"]])
+    all.agg.funs <- c("min", "mean", "max")
+    agg.fun <- tolower(as.character(if_null_then(imp.fun, A[["agg.fun"]], all.agg.funs)))
+    agg.fun <- match_arg(agg.fun, all.agg.funs, several.ok = TRUE)
     
     #Setup output object
     out.names <- c("Imputation.Balance", 
@@ -172,7 +139,16 @@ base.bal.tab.imp <- function(X, which.imp = NA, imp.summary = getOption("cobalt_
     
     if (imp.summary || !A$quick) {
         out[["Balance.Across.Imputations"]] <- balance.summary(out[["Imputation.Balance"]], 
-                                                               agg.funs = if_null_then(imp.fun, c("min", "mean", "max")))
+                                                               agg.funs = agg.fun)
+        
+        if (length(agg.fun) == 1) {
+            out <- c(out, threshold.summary(compute = attr(out[["Imputation.Balance"]][[1]][["Balance"]], "compute"),
+                                            thresholds = attr(out[["Imputation.Balance"]][[1]][["Balance"]], "thresholds"),
+                                            no.adj = !attr(out[["Imputation.Balance"]][[1]], "print.options")$disp.adj,
+                                            balance.table = out[["Balance.Across.Imputations"]],
+                                            weight.names = attr(out[["Imputation.Balance"]][[1]], "print.options")$weight.names,
+                                            agg.fun = agg.fun))
+        }
         
         observations <- lapply(out[["Imputation.Balance"]], function(x) x[["Observations"]])
         
@@ -183,7 +159,7 @@ base.bal.tab.imp <- function(X, which.imp = NA, imp.summary = getOption("cobalt_
     attr(out, "print.options") <- c(attr(out[["Imputation.Balance"]][[1]], "print.options"),
                                     list(which.imp = which.imp,
                                          imp.summary = imp.summary,
-                                         imp.fun = imp.fun))
+                                         imp.fun = agg.fun))
     class(out) <- c("bal.tab.imp", "bal.tab")
     
     return(out)
@@ -191,7 +167,7 @@ base.bal.tab.imp <- function(X, which.imp = NA, imp.summary = getOption("cobalt_
 base.bal.tab.multi <- function(X, pairwise = TRUE, which.treat, multi.summary = getOption("cobalt_multi.summary"), ...) {
     A <- list(...)
     
-    X$treat <- process_treat(X$treat)
+    # X$treat <- process_treat(X$treat)
     
     #Preparations
     
@@ -207,15 +183,13 @@ base.bal.tab.multi <- function(X, pairwise = TRUE, which.treat, multi.summary =
     }
     
     if (is_null(multi.summary)) {
-        multi.summary <- is_not_null(which.treat) && (anyNA(which.treat) || !is_(which.treat, c("character", "numeric")) ||
-                                                          (is.numeric(which.treat) && !any(which.treat %in% seq_len(nlevels(X$treat)))) ||
-                                                          (is.character(which.treat) && !any(which.treat %in% levels(X$treat))))
+        multi.summary <- is_not_null(which.treat) && anyNA(which.treat)
     }
     
     #Treat is a factor variable
     if (is_null(X$focal)) {
-        if (pairwise) treat.combinations <- combn(levels(X$treat), 2, simplify = FALSE)
-        else treat.combinations <- lapply(levels(X$treat), function(x) c(x, "All"))
+        if (pairwise) treat.combinations <- combn(treat_names(X$treat), 2, simplify = FALSE)
+        else treat.combinations <- lapply(treat_names(X$treat), function(x) c(x, "All"))
     }
     else {
         if (length(X$focal) > 1) stop("'focal' must be a vector of length 1 containing the name or index of the focal treatment group.", call. = FALSE)
@@ -232,11 +206,12 @@ base.bal.tab.multi <- function(X, pairwise = TRUE, which.treat, multi.summary =
     }
     
     #Setup output object
-    out.names <- c("Pair.Balance", 
-                   "Balance.Across.Pairs", 
-                   "Observations", 
-                   "call")
-    out <- make_list(out.names)
+    # out.names <- c("Pair.Balance", 
+    #                "Balance.Across.Pairs", 
+    #                "Observations", 
+    #                "call")
+    # out <- make_list(out.names)
+    out <- list()
     
     if ("mean.diffs" %in% X$stats) {
         C <- do.call("get.C2", c(X, A[names(A) %nin% names(X)]), quote = TRUE)
@@ -254,7 +229,9 @@ base.bal.tab.multi <- function(X, pairwise = TRUE, which.treat, multi.summary =
     
     if (pairwise || is_not_null(X$focal)) {
         balance.tables <- lapply(treat.combinations, function(t) {
-            X_t <- assign.X.class(subset_X(X, X$treat %in% t))
+            X_t <- subset_X(X, X$treat %in% treat_vals(X$treat)[t])
+            X_t$treat <- process_treat(X_t$treat)
+            X_t <- assign.X.class(X_t)
             X_t$call <- NULL
             do.call("base.bal.tab", c(list(X_t), A[names(A) %nin% names(X_t)]), quote = TRUE)
         })
@@ -265,9 +242,10 @@ base.bal.tab.multi <- function(X, pairwise = TRUE, which.treat, multi.summary =
             n <- length(X$treat)
             X_t <- X
             X_t$call <- NULL
-            X_t <- subset_X(X_t, c(seq_len(n), which(X$treat == t[1])))
-            X_t$treat <- factor(c(rep("All", n), rep(t[1], sum(X$treat == t[1]))), nmax = 2,
-                                levels = c("All", t[1]))
+            X_t <- subset_X(X_t, c(seq_len(n), which(X$treat == treat_vals(X$treat)[t[1]])))
+            X_t$treat <- factor(rep(0:1, times = c(n, sum(X$treat == treat_vals(X$treat)[t[1]]))),
+                                levels = c(0, 1), labels = c("All", t[1]))
+            # if (is_not_null(X_t$weights)) X_t$weights[X_t$treat == "All",] <- 1 #Uncomment to compare each group to unweighted dist.
             X_t <- assign.X.class(X_t)
             do.call("base.bal.tab", c(list(X_t), A[names(A) %nin% names(X_t)]), quote = TRUE)
         })
@@ -282,13 +260,21 @@ base.bal.tab.multi <- function(X, pairwise = TRUE, which.treat, multi.summary =
     if ((multi.summary || !A$quick) && is_null(X$imp)) {
         out[["Balance.Across.Pairs"]] <- balance.summary(balance.tables, 
                                                          agg.funs = "max")
+        
+        out <- c(out, threshold.summary(compute = attr(out[["Pair.Balance"]][[1]][["Balance"]], "compute"),
+                                        thresholds = attr(out[["Pair.Balance"]][[1]][["Balance"]], "thresholds"),
+                                        no.adj = !attr(out[["Pair.Balance"]][[1]], "print.options")$disp.adj,
+                                        balance.table = out[["Balance.Across.Pairs"]],
+                                        weight.names = attr(out[["Pair.Balance"]][[1]], "print.options")$weight.names,
+                                        agg.fun = "max"))
+        
         out[["Observations"]] <- samplesize.multi(balance.tables, treat_names(X$treat), X$focal)
     }
     
     out[["call"]] <- X$call
     
     attr(out, "print.options") <- c(attr(out[["Pair.Balance"]][[1]], "print.options"),
-                                    list(treat_names_multi = treat_vals(X$treat),
+                                    list(treat_vals_multi = treat_vals(X$treat),
                                          which.treat = which.treat,
                                          multi.summary = multi.summary,
                                          pairwise = pairwise))
@@ -308,7 +294,7 @@ base.bal.tab.msm <- function(X, which.time, msm.summary = getOption("cobalt_msm.
     
     A <- list(...)
     
-    X$treat.list <- process_treat.list(X$treat)
+    # X$treat.list <- process_treat.list(X$treat)
     
     #Preparations
     
@@ -328,11 +314,12 @@ base.bal.tab.msm <- function(X, which.time, msm.summary = getOption("cobalt_msm.
     }
     
     #Setup output object
-    out.names <- c("Time.Balance", 
-                   "Balance.Across.Times", 
-                   "Observations", 
-                   "call")
-    out <- make_list(out.names)
+    # out.names <- c("Time.Balance", 
+    #                "Balance.Across.Times", 
+    #                "Observations", 
+    #                "call")
+    # out <- make_list(out.names)
+    out <- list()
     
     out[["Time.Balance"]] <- make_list(length(X$covs.list))
     
@@ -362,6 +349,14 @@ base.bal.tab.msm <- function(X, which.time, msm.summary = getOption("cobalt_msm.
         out[["Balance.Across.Times"]] <- balance.summary(out[["Time.Balance"]],
                                                          agg.funs = "max",
                                                          include.times = TRUE)
+        
+        out <- c(out, threshold.summary(compute = attr(out[["Time.Balance"]][[1]][["Balance"]], "compute"),
+                                        thresholds = attr(out[["Time.Balance"]][[1]][["Balance"]], "thresholds"),
+                                        no.adj = !attr(out[["Time.Balance"]][[1]], "print.options")$disp.adj,
+                                        balance.table = out[["Balance.Across.Times"]],
+                                        weight.names = attr(out[["Time.Balance"]][[1]], "print.options")$weight.names,
+                                        agg.fun = "max"))
+        
         out[["Observations"]] <- lapply(out[["Time.Balance"]], function(x) x$Observations)
     }
     
@@ -394,14 +389,17 @@ base.bal.tab.cluster <- function(X, which.cluster, cluster.summary = getOption("
         cluster.summary <- is_not_null(which.cluster) && anyNA(which.cluster)
     }
     
-    cluster.fun <- if_null_then(cluster.fun, A[["agg.fun"]])
+    all.agg.funs <- c("min", "mean", "max")
+    agg.fun <- tolower(as.character(if_null_then(cluster.fun, A[["agg.fun"]], all.agg.funs)))
+    agg.fun <- match_arg(agg.fun, all.agg.funs, several.ok = TRUE)
     
     #Setup output object
-    out.names <- c("Cluster.Balance", 
-                   "Balance.Across.Clusters", 
-                   "Observations", 
-                   "call")
-    out <- make_list(out.names)
+    # out.names <- c("Cluster.Balance", 
+    #                "Balance.Across.Clusters", 
+    #                "Observations", 
+    #                "call")
+    # out <- make_list(out.names)
+    out <- list()
     
     #Get list of bal.tabs for each imputation
     out[["Cluster.Balance"]] <- lapply(levels(cluster), function(cl) {
@@ -416,7 +414,17 @@ base.bal.tab.cluster <- function(X, which.cluster, cluster.summary = getOption("
     
     if ((cluster.summary || !A$quick) && is_null(X$covs.list) && get.treat.type(X$treat) != "multinomial" && is_null(X$imp)) {
         out[["Balance.Across.Clusters"]] <- balance.summary(out[["Cluster.Balance"]], 
-                                                            agg.funs = if_null_then(cluster.fun, c("min", "mean", "max")))
+                                                            agg.funs = if_null_then(agg.fun, c("min", "mean", "max")))
+        
+        if (length(agg.fun) == 1) {
+            out <- c(out, threshold.summary(compute = attr(out[["Cluster.Balance"]][[1]][["Balance"]], "compute"),
+                                            thresholds = attr(out[["Cluster.Balance"]][[1]][["Balance"]], "thresholds"),
+                                            no.adj = !attr(out[["Cluster.Balance"]][[1]], "print.options")$disp.adj,
+                                            balance.table = out[["Balance.Across.Clusters"]],
+                                            weight.names = attr(out[["Cluster.Balance"]][[1]], "print.options")$weight.names,
+                                            agg.fun = agg.fun))
+        }
+        
         observations <- lapply(out[["Cluster.Balance"]], function(x) x[["Observations"]])
         
         out[["Observations"]] <- samplesize.across.clusters(observations)
@@ -427,14 +435,14 @@ base.bal.tab.cluster <- function(X, which.cluster, cluster.summary = getOption("
     attr(out, "print.options") <- c(attr(out[["Cluster.Balance"]][[1]], "print.options"),
                                     list(which.cluster = which.cluster,
                                          cluster.summary = cluster.summary,
-                                         cluster.fun = cluster.fun))
+                                         cluster.fun = agg.fun))
     class(out) <- c("bal.tab.cluster", "bal.tab")
     
     return(out)
 }
 
 #NEEDS UPDATING with STATS
-base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, binary, imbalanced.only = getOption("cobalt_imbalanced.only", FALSE), un = getOption("cobalt_un", FALSE), disp = NULL, disp.subclass = getOption("cobalt_disp.subclass", FALSE), disp.bal.tab = getOption("cobalt_disp.bal.tab", TRUE), abs = FALSE, quick = TRUE, ...) {
+base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, binary, imbalanced.only = getOption("cobalt_imbalanced.only", FALSE), un = getOption("cobalt_un", FALSE), disp = NULL, which.subclass = NA, subclass.summary = getOption("cobalt_subclass.summary"), disp.bal.tab = getOption("cobalt_disp.bal.tab", TRUE), disp.call = getOption("cobalt_disp.call", TRUE), abs = FALSE, quick = TRUE, ...) {
     #Preparations
     
     A <- clear_null(list(...))
@@ -451,6 +459,19 @@ base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, bi
         if (missing(binary)) binary <- getOption("cobalt_binary", "std")
     }
     
+    subclass <- factor(X$subclass)
+    
+    if (missing(which.subclass)) {
+        if (isTRUE(A[["disp.subclass"]])) which.subclass <- seq_len(nlevels(subclass))
+        else which.subclass <- NA
+    }
+    
+    if (is_null(subclass.summary)) {
+        subclass.summary <- is_not_null(which.subclass) && 
+            (anyNA(which.subclass) || !is.numeric(which.subclass) || 
+                 (is.numeric(which.subclass) && !any(which.subclass %in% seq_len(nlevels(subclass)))))
+    }
+    
     no.adj <- FALSE
     
     if (is_null(X$s.weights)) {
@@ -461,25 +482,11 @@ base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, bi
     
     #Actions
     
-    out.names <- c("Subclass.Balance", "Balance.Across.Subclass", 
-                   expand.grid_string(c("Balanced", "Max.Imbalance"),
-                                      X$stats,
-                                      "Subclass",
-                                      collapse = "."), 
-                   "Observations", "call")
-    out <- make_list(out.names)
+    out <- list()
     
     C <- do.call("get.C2", c(X, A[names(A) %nin% names(X)], list(int = int, poly = poly)), quote = TRUE)
     co.names <- attr(C, "co.names")
     
-    out[["Observations"]] <- samplesize(treat = X$treat, 
-                                        type = type,
-                                        weights = NULL, 
-                                        subclass = X$subclass,
-                                        s.weights = X$s.weights, 
-                                        method = X$method, 
-                                        discarded = X$discarded)
-    
     out[["Subclass.Balance"]] <- do.call("balance.table.subclass", 
                                          c(list(C, type = type, 
                                                 weights = NULL, 
@@ -493,45 +500,48 @@ base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, bi
                                                 s.weights = X$s.weights, 
                                                 abs = abs, 
                                                 quick = quick), A), quote = TRUE)
-    if (type == "bin") {
-        out[["Balance.Across.Subclass"]] <- do.call("balance.table", 
-                                                    c(list(C, 
-                                                           type = type, 
-                                                           weights = data.frame(Adj = strata2weights(X$subclass, X$treat, X$estimand)), 
-                                                           treat = X$treat, 
-                                                           s.d.denom = X$s.d.denom[1], 
-                                                           s.weights = X$s.weights, 
-                                                           continuous = continuous, 
-                                                           binary = binary, 
-                                                           thresholds = X$thresholds,
-                                                           un = un, 
-                                                           disp = disp,
-                                                           stats = X$stats, 
-                                                           abs = abs, 
-                                                           no.adj = FALSE, quick = quick, 
-                                                           s.d.denom.list = X$s.d.denom.list), A), quote = TRUE)
-    }
-    else if (type == "cont") {
-        out[["Balance.Across.Subclass"]] <- do.call("balance.table.across.subclass.cont", 
-                                                    c(list(do.call("balance.table", c(list(C, 
-                                                                                           type = type, 
-                                                                                           weights = NULL,
-                                                                                           treat = X$treat, 
-                                                                                           s.d.denom = X$s.d.denom[1], 
-                                                                                           s.weights = X$s.weights, 
-                                                                                           continuous = continuous, 
-                                                                                           binary = binary, 
-                                                                                           thresholds = X$thresholds,
-                                                                                           un = un, 
-                                                                                           disp = disp,
-                                                                                           stats = X$stats, 
-                                                                                           abs = abs, 
-                                                                                           no.adj = TRUE, 
-                                                                                           quick = quick, 
-                                                                                           s.d.denom.list = X$s.d.denom.list), A), quote = TRUE), 
-                                                           balance.table.subclass.list = out[["Subclass.Balance"]], 
-                                                           subclass.obs = out[["Observations"]], 
-                                                           r.threshold = X$thresholds[["correlations"]]), A), quote = TRUE)
+    
+    if (subclass.summary || !quick) {
+        if (type == "bin") {
+            out[["Balance.Across.Subclass"]] <- do.call("balance.table", 
+                                                        c(list(C, 
+                                                               type = type, 
+                                                               weights = data.frame(Adj = strata2weights(X$subclass, X$treat, X$estimand)), 
+                                                               treat = X$treat, 
+                                                               s.d.denom = X$s.d.denom[1], 
+                                                               s.weights = X$s.weights, 
+                                                               continuous = continuous, 
+                                                               binary = binary, 
+                                                               thresholds = X$thresholds,
+                                                               un = un, 
+                                                               disp = disp,
+                                                               stats = X$stats, 
+                                                               abs = abs, 
+                                                               no.adj = FALSE, quick = quick, 
+                                                               s.d.denom.list = X$s.d.denom.list), A), quote = TRUE)
+        }
+        else if (type == "cont") {
+            out[["Balance.Across.Subclass"]] <- do.call("balance.table.across.subclass.cont", 
+                                                        c(list(do.call("balance.table", c(list(C, 
+                                                                                               type = type, 
+                                                                                               weights = NULL,
+                                                                                               treat = X$treat, 
+                                                                                               s.d.denom = X$s.d.denom[1], 
+                                                                                               s.weights = X$s.weights, 
+                                                                                               continuous = continuous, 
+                                                                                               binary = binary, 
+                                                                                               thresholds = X$thresholds,
+                                                                                               un = un, 
+                                                                                               disp = disp,
+                                                                                               stats = X$stats, 
+                                                                                               abs = abs, 
+                                                                                               no.adj = TRUE, 
+                                                                                               quick = quick, 
+                                                                                               s.d.denom.list = X$s.d.denom.list), A), quote = TRUE), 
+                                                               balance.table.subclass.list = out[["Subclass.Balance"]], 
+                                                               subclass.obs = out[["Observations"]], 
+                                                               r.threshold = X$thresholds[["correlations"]]), A), quote = TRUE)
+        }
     }
     
     #Reassign disp... and ...threshold based on balance table output
@@ -552,12 +562,17 @@ base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, bi
             out[[paste.("Max.Imbalance", s, "Subclass")]] <- data.frame(do.call("rbind", max.imbal.list), 
                                                                         row.names = paste("Subclass", levels(X$subclass)))
         }
-        else {
-            out[[paste.("Balanced", s, "Subclass")]] <- NULL
-            out[[paste.("Max.Imbalance", s, "Subclass")]] <- NULL
-        }
     }
     
+    out[["Observations"]] <- samplesize(treat = X$treat, 
+                                        type = type,
+                                        weights = NULL, 
+                                        subclass = X$subclass,
+                                        s.weights = X$s.weights, 
+                                        method = X$method, 
+                                        discarded = X$discarded)
+    
+    
     out[["call"]] <- X$call
     attr(out, "print.options") <- list(thresholds = thresholds,
                                        imbalanced.only = imbalanced.only,
@@ -565,8 +580,10 @@ base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, bi
                                        disp=disp,
                                        compute = compute, 
                                        disp.adj = !no.adj, 
-                                       disp.subclass = disp.subclass,
+                                       which.subclass = which.subclass,
+                                       subclass.summary = subclass.summary,
                                        disp.bal.tab = disp.bal.tab, 
+                                       disp.call = disp.call,
                                        abs = abs,
                                        continuous = continuous,
                                        binary = binary,
@@ -574,7 +591,7 @@ base.bal.tab.subclass <- function(X, type, int = FALSE, poly = 1, continuous, bi
                                        treat_names = treat_vals(X$treat),
                                        type = type,
                                        co.names = co.names)
-    class(out) <- c("bal.tab.subclass", paste.("bal.tab", type), "bal.tab")
+    class(out) <- c("bal.tab.subclass", "bal.tab")
     
     return(out)
 }
@@ -584,4 +601,4 @@ base.bal.tab.subclass.binary <- function(X, ...) {
 base.bal.tab.subclass.cont <- function(X, ...) {
     stop("Subclasses are not yet compatible with continuous treatments.", call. = FALSE)
     base.bal.tab.subclass(X, type = "cont", ...)
-}
\ No newline at end of file
+}
